"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const fixednum_1 = require("./fixednum");
class PerpAccount {
    constructor(decoded) {
        Object.assign(this, decoded);
    }
    getAverageEntryPrice(events) { }
    getPnl(perpMarketInfo, price) {
        return fixednum_1.I80F48.fromI64(this.basePosition.mul(perpMarketInfo.baseLotSize))
            .mul(price)
            .add(this.quotePosition);
    }
    getUnsettledFunding(perpMarketCache) {
        if (this.basePosition.isNeg()) {
            return fixednum_1.I80F48.fromI64(this.basePosition).mul(perpMarketCache.shortFunding.sub(this.shortSettledFunding));
        }
        else {
            return fixednum_1.I80F48.fromI64(this.basePosition).mul(perpMarketCache.longFunding.sub(this.longSettledFunding));
        }
    }
    /**
     * Return the quote position after adjusting for unsettled funding
     */
    getQuotePosition(perpMarketCache) {
        return this.quotePosition.sub(this.getUnsettledFunding(perpMarketCache));
    }
    simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, baseChange) {
        const newBase = this.basePosition.add(baseChange);
        let health = this.quotePosition.sub(fixednum_1.I80F48.fromI64(baseChange.mul(perpMarketInfo.baseLotSize)).mul(price));
        if (newBase.gt(_1.ZERO_BN)) {
            health = health.add(fixednum_1.I80F48.fromI64(newBase.mul(perpMarketInfo.baseLotSize))
                .mul(price)
                .mul(assetWeight));
        }
        else {
            health = health.add(fixednum_1.I80F48.fromI64(newBase.mul(perpMarketInfo.baseLotSize))
                .mul(price)
                .mul(liabWeight));
        }
        return health;
    }
    getHealth(perpMarketInfo, price, assetWeight, liabWeight, longFunding, shortFunding) {
        const bidsHealth = this.simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, this.openOrders.bidsQuantity);
        const asksHealth = this.simPositionHealth(perpMarketInfo, price, assetWeight, liabWeight, this.openOrders.asksQuantity.neg());
        const health = bidsHealth.lt(asksHealth) ? bidsHealth : asksHealth;
        let x;
        if (this.basePosition.gt(_1.ZERO_BN)) {
            x = health.sub(longFunding
                .sub(this.longSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
        else {
            x = health.add(shortFunding
                .sub(this.shortSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
        return x;
    }
    getLiabsVal(perpMarketInfo, price, shortFunding, longFunding) {
        let liabsVal = fixednum_1.ZERO_I80F48;
        if (this.basePosition.lt(_1.ZERO_BN)) {
            liabsVal = liabsVal.add(fixednum_1.I80F48.fromI64(this.basePosition.mul(perpMarketInfo.baseLotSize)).mul(price));
        }
        let realQuotePosition = this.quotePosition;
        if (this.basePosition.gt(_1.ZERO_BN)) {
            realQuotePosition = this.quotePosition.sub(longFunding
                .sub(this.longSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
        else if (this.basePosition.lt(_1.ZERO_BN)) {
            realQuotePosition = this.quotePosition.sub(shortFunding
                .sub(this.shortSettledFunding)
                .mul(fixednum_1.I80F48.fromI64(this.basePosition)));
        }
        if (realQuotePosition.lt(fixednum_1.ZERO_I80F48)) {
            liabsVal = liabsVal.add(realQuotePosition);
        }
        return liabsVal.neg();
    }
}
exports.default = PerpAccount;
//# sourceMappingURL=PerpAccount.js.map